// Grounder32.cpp : implementation file
//

#include "stdafx.h"
#include "resource.h"   
#include "grounder.h"
#include "Grounder32.h"
#include "help.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CGrounder32 dialog


CGrounder32::CGrounder32(CWnd* pParent /*=NULL*/)
	: CDialog(CGrounder32::IDD, pParent)
{
	//{{AFX_DATA_INIT(CGrounder32)
	m_nGPE = 10;
	m_nPPE = 10;
	m_bGroundOn = FALSE;
	m_bPowerOn = FALSE;
	m_bGPRelievePadsVias = FALSE;
	m_bPPRelievePadsVias = FALSE;
	//}}AFX_DATA_INIT
	m_pOutputFile = NULL;
	m_pInputFile = NULL;
	m_bInputFileOpened = false;
	m_sInputFileName.Empty();
	m_sOutputFileName.Empty();
	m_nPowerSharedLayer = 2;
	m_nGroundSharedLayer = 1;
}


void CGrounder32::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CGrounder32)
	DDX_Text(pDX, IDC_GROUND_ENLARGEMENT, m_nGPE);
	DDV_MinMaxInt(pDX, m_nGPE, 0, 100);
	DDX_Text(pDX, IDC_POWER_ENLARGEMENT, m_nPPE);
	DDV_MinMaxInt(pDX, m_nPPE, 0, 100);
	DDX_Check(pDX, IDC_GROUND_ON, m_bGroundOn);
	DDX_Check(pDX, IDC_POWER_ON, m_bPowerOn);
	DDX_Check(pDX, IDC_GP_RELIEVE_PADS_VIAS, m_bGPRelievePadsVias);
	DDX_Check(pDX, IDC_PP_RELIEVE_PADS_VIAS, m_bPPRelievePadsVias);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CGrounder32, CDialog)
	//{{AFX_MSG_MAP(CGrounder32)
	ON_BN_CLICKED(ID_OPEN_PCB, OnOpenPcb)
	ON_BN_CLICKED(ID_RUN, OnRun)
	ON_BN_CLICKED(IDC_GROUND_BOTTOM, OnGroundBottom)
	ON_BN_CLICKED(IDC_GROUND_MID1, OnGroundMid1)
	ON_BN_CLICKED(IDC_GROUND_MID2, OnGroundMid2)
	ON_BN_CLICKED(IDC_GROUND_MID3, OnGroundMid3)
	ON_BN_CLICKED(IDC_GROUND_MID4, OnGroundMid4)
	ON_BN_CLICKED(IDC_GROUND_TOP, OnGroundTop)
	ON_BN_CLICKED(IDC_POWER_BOTTOM, OnPowerBottom)
	ON_BN_CLICKED(IDC_POWER_MID1, OnPowerMid1)
	ON_BN_CLICKED(IDC_POWER_MID2, OnPowerMid2)
	ON_BN_CLICKED(IDC_POWER_MID3, OnPowerMid3)
	ON_BN_CLICKED(IDC_POWER_MID4, OnPowerMid4)
	ON_BN_CLICKED(IDC_POWER_TOP, OnPowerTop)
	ON_BN_CLICKED(ID_BUTTON_HELP, OnButtonHelp)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CGrounder32 message handlers

void CGrounder32::OnOpenPcb() 
{
	if(m_pInputFile != NULL) 
	{
		if(AfxMessageBox("You have a PCB file open already.  OK to close?", MB_YESNO, 0) == IDNO)
			return;
		else
			TidyUp(m_pInputFile);
	}  
	
	static char BASED_CODE fileDialogFilter[] =
	"PCB files (*.pcb)|*.pcb|All Files (*.*)|*.*||"; 
	
	const char fileDialogExt[] = "pcb"; 
	
	CFileDialog fileDialog(TRUE, fileDialogExt, NULL, OFN_FILEMUSTEXIST, fileDialogFilter );
		 
	if( fileDialog.DoModal() == IDOK )
	{   
		try
		{
			m_pInputFile = new CStdioFile((const char*)fileDialog.GetPathName(), CFile::modeRead);
		}
		catch(CFileException* p_ex)
		{
			char s[100];
			sprintf(s, "Error opening file for reading.  Code: %d", p_ex->m_cause);
			AfxMessageBox(s, MB_OK, 0);
			GetDlgItem(ID_RUN)->EnableWindow(FALSE);
			TidyUp(m_pInputFile);
			return;
		}

		char buffer[81];
		CString cs(m_pInputFile->ReadString(buffer, 80));    // read first line
		
		if(!((cs == "PCB FILE 4\n") || (cs == "PCB FILE 5\n")))  // check first line
		{
			char s[100];
			sprintf(s, "Input file is not an Easytrax or Autotrax file.");
			AfxMessageBox(s, MB_OK | MB_ICONEXCLAMATION, 0);
			GetDlgItem(ID_RUN)->EnableWindow(FALSE);
			TidyUp(m_pInputFile);
			return;
		}

		m_sInputFileName = fileDialog.GetPathName();
		GetDlgItem(ID_RUN)->EnableWindow(TRUE);
   }
	return;
}

void CGrounder32::OnRun() 
{
	UpdateData(TRUE);
	if(!GetOutputFile())
		return;

	BeginWaitCursor();
	CopyFile();
	EndWaitCursor();
	
	TidyUp(m_pOutputFile);
}

BOOL CGrounder32::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
	GetDlgItem(ID_RUN)->EnableWindow(FALSE);
	CheckDlgButton(IDC_GROUND_ON, BST_CHECKED);
	CheckRadioButton(IDC_GROUND_TOP, IDC_GROUND_BOTTOM, IDC_GROUND_TOP);
	CheckRadioButton(IDC_POWER_TOP, IDC_POWER_BOTTOM, IDC_POWER_MID1);
	
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CGrounder32::TidyUp(CStdioFile*& pFile)
{
	if(pFile == NULL)
		return;
	pFile->Close();    // tidy up
	delete pFile; 
	pFile = NULL;   
}

CGrounder32::~CGrounder32()
{
	TidyUp(m_pInputFile);
	TidyUp(m_pOutputFile);
}

void CGrounder32::OnGroundTop() { m_nGroundSharedLayer = 1; return; } 
void CGrounder32::OnGroundMid1() { m_nGroundSharedLayer = 2; return; }  
void CGrounder32::OnGroundMid2() { m_nGroundSharedLayer = 3; return; }  
void CGrounder32::OnGroundMid3() { m_nGroundSharedLayer = 4; return; }  
void CGrounder32::OnGroundMid4() { m_nGroundSharedLayer = 5; return; }  
void CGrounder32::OnGroundBottom() { m_nGroundSharedLayer = 6; return; }  
void CGrounder32::OnPowerTop() { m_nPowerSharedLayer = 1; return; }  
void CGrounder32::OnPowerMid1() { m_nPowerSharedLayer = 2; return; }  
void CGrounder32::OnPowerMid2() { m_nPowerSharedLayer = 3; return; }  
void CGrounder32::OnPowerMid3() { m_nPowerSharedLayer = 4; return; }  
void CGrounder32::OnPowerMid4() { m_nPowerSharedLayer = 5; return; }  
void CGrounder32::OnPowerBottom() { m_nPowerSharedLayer = 6; return; }  

bool CGrounder32::GetOutputFile()
{
	if(m_pInputFile == NULL)
	{
		AfxMessageBox("No input file chosen yet");
		return false;
	}

	m_sOutputFileName = m_sInputFileName;  
	int pos = m_sOutputFileName.ReverseFind('\\');
	m_sOutputFileName.SetAt(pos+1, '_');
	
	if(m_pOutputFile)           			// if the file is somehow already open
		TidyUp(m_pOutputFile);        
		
	try
	{
		m_pOutputFile = new CStdioFile((const char*)m_sOutputFileName, CFile::modeWrite | CFile::modeCreate); 
	}
	catch (CFileException* p_ex)
	{
		char s[100];
		sprintf(s, "Error opening file for writing.  Code: %d", p_ex->m_cause);
		AfxMessageBox(s, MB_OK, 0);
		GetDlgItem(ID_RUN)->EnableWindow(FALSE);
		return false;
	}
	   	
	return true;
}

void CGrounder32::CopyFile()
{
	ASSERT_VALID(m_pInputFile);
	m_pInputFile->SeekToBegin();
	char buffer[41];
	CString cs, ws;
	int count = 0;
	int p1, p2, p3, p4, p6, p7;	// storage for other data parameters
	int nXPos, nYPos, nXSize, nYSize, nShape, nHole, nPlane, nLayer;
	int dx, dy, dd;

	do
	{
		cs = CString(m_pInputFile->ReadString(buffer, 40));    // read next line  
		m_pOutputFile->WriteString(buffer);
		if(*buffer == '\0')
			break;
		if(cs.Find("ENDPCB") >= 0)
			break;
		if(count)
			count--;
		else
		{
			if(cs.Left(4) == "COMP")
				count = 6;
			else if((cs.Left(2) == "FA"))    // arcs
			{
				m_pInputFile->ReadString(buffer, 40);
				m_pOutputFile->WriteString(buffer);
				sscanf(buffer, "%d %d %d %d %d %d", &p1, &p2, &p3, &p4, &nXSize, &p6);
				if(p6 == m_nGroundSharedLayer && m_bGroundOn)
				{
					m_pOutputFile->WriteString("FA\n");
					wsprintf(buffer, "%d %d %d %d %d %d\n", p1, p2, p3, p4, nXSize + 2 * m_nGPE, 9);
					m_pOutputFile->WriteString(buffer);
				}				
				if(p6 == m_nPowerSharedLayer && m_bPowerOn)
				{
					m_pOutputFile->WriteString("FA\n");
					wsprintf(buffer, "%d %d %d %d %d %d\n", p1, p2, p3, p4, nXSize + 2 * m_nPPE, 10);
					m_pOutputFile->WriteString(buffer);
				}				
				 
			} 
			else if((cs.Left(2) == "FT"))     // tracks
			{		
				m_pInputFile->ReadString(buffer, 40);
				m_pOutputFile->WriteString(buffer);
				sscanf(buffer, "%d %d %d %d %d %d %d", &p1, &p2, &p3, &p4, &nXSize, &p6, &p7);
				if(p6 == m_nGroundSharedLayer && m_bGroundOn)
				{
					m_pOutputFile->WriteString("FT\n");
					wsprintf(buffer, "%d %d %d %d %d %d %d\n", p1, p2, p3, p4, nXSize + 2 * m_nGPE, 9, p7);
					m_pOutputFile->WriteString(buffer);
				}				
				if(p6 == m_nPowerSharedLayer && m_bPowerOn)
				{
					m_pOutputFile->WriteString("FT\n");
					wsprintf(buffer, "%d %d %d %d %d %d %d\n", p1, p2, p3, p4, nXSize + 2 * m_nPPE, 10, p7);
					m_pOutputFile->WriteString(buffer);
				}				
			}
			else if((cs.Left(2) == "FV"))     // vias
			{		
				m_pInputFile->ReadString(buffer, 40);
				m_pOutputFile->WriteString(buffer);
				sscanf(buffer, "%d %d %d %d", &p1, &p2, &nXSize, &p3);
				if(m_bGroundOn && m_bGPRelievePadsVias)
				{
					m_pOutputFile->WriteString("FA\n");
					wsprintf(buffer, "%d %d %d %d %d %d\n", p1, p2, ((nXSize / 2) + m_nGPE) / 2, 15, (nXSize / 2) + m_nGPE, 9);
					m_pOutputFile->WriteString(buffer);
				}				
				if(m_bPowerOn && m_bPPRelievePadsVias)
				{
					m_pOutputFile->WriteString("FA\n");
					wsprintf(buffer, "%d %d %d %d %d %d\n", p1, p2, ((nXSize / 2) + m_nPPE) / 2, 15, (nXSize / 2) + m_nPPE, 10);
					m_pOutputFile->WriteString(buffer);
				}				
			}
			else if((cs.Left(2) == "FP"))     // pads
			{		
				m_pInputFile->ReadString(buffer, 40);
				m_pOutputFile->WriteString(buffer);		// copy the original
				sscanf(buffer, "%d %d %d %d %d %d %d %d", &nXPos, &nYPos, &nXSize, &nYSize, &nShape, &nHole, &nPlane, &nLayer);
				m_pInputFile->ReadString(buffer, 40);	// don't forget the name of the pad!
				m_pOutputFile->WriteString(buffer);		// copy the original
				if((m_bGroundOn || m_bPowerOn) && (nLayer == 13 || nLayer == m_nGroundSharedLayer || nLayer == m_nPowerSharedLayer) && (m_bGPRelievePadsVias || m_bPPRelievePadsVias))
				{
					switch(nShape)
					{
					case 1:
						if(m_bGPRelievePadsVias && m_bGroundOn && (nLayer == m_nGroundSharedLayer || nLayer == 13) && ((nPlane == 1) || (nPlane == 4) || (nPlane == 5))) {
							m_pOutputFile->WriteString("FA\n");	// draw arcs for circular pads
							wsprintf(buffer, "%d %d %d %d %d %d\n", nXPos, nYPos, (nXSize + m_nGPE) / 2, 15, m_nGPE, 9);
							m_pOutputFile->WriteString(buffer);
						}
						if(m_bPPRelievePadsVias && m_bPowerOn && (nLayer == m_nPowerSharedLayer || nLayer == 13) && ((nPlane == 1) || (nPlane == 2) || (nPlane == 3))) {
							m_pOutputFile->WriteString("FA\n");	// draw arcs for circular pads
							wsprintf(buffer, "%d %d %d %d %d %d\n", nXPos, nYPos, (nXSize + m_nPPE) / 2, 15, m_nPPE, 10);
							m_pOutputFile->WriteString(buffer);
						}
						break;
					case 2:
						if(m_bGPRelievePadsVias && m_bGroundOn && (nLayer == m_nGroundSharedLayer || nLayer == 13) && ((nPlane == 1) || (nPlane == 4) || (nPlane == 5))) {
							dx = (nXSize + m_nGPE) / 2;
							dy = (nYSize + m_nGPE) / 2;
							m_pOutputFile->WriteString("FT\n"); 	// draw tracks for rectangular pads
							wsprintf(buffer, "%d %d %d %d %d %d\n", nXPos - dx, nYPos + dy, nXPos + dx, nYPos + dy, m_nGPE, 9);
							m_pOutputFile->WriteString(buffer);
							m_pOutputFile->WriteString("FT\n"); 
							wsprintf(buffer, "%d %d %d %d %d %d\n", nXPos + dx, nYPos + dy, nXPos + dx, nYPos - dy, m_nGPE, 9);
							m_pOutputFile->WriteString(buffer);
							m_pOutputFile->WriteString("FT\n"); 
							wsprintf(buffer, "%d %d %d %d %d %d\n", nXPos + dx, nYPos - dy, nXPos - dx, nYPos - dy, m_nGPE, 9);
							m_pOutputFile->WriteString(buffer);
							m_pOutputFile->WriteString("FT\n"); 
							wsprintf(buffer, "%d %d %d %d %d %d\n", nXPos - dx, nYPos - dy, nXPos - dx, nYPos + dy, m_nGPE, 9);
							m_pOutputFile->WriteString(buffer);
						}
						if(m_bPPRelievePadsVias && m_bPowerOn && (nLayer == m_nPowerSharedLayer || nLayer == 13) && ((nPlane == 1) || (nPlane == 2) || (nPlane == 3))) {
							dx = (nXSize + m_nPPE) / 2;
							dy = (nYSize + m_nPPE) / 2;
							m_pOutputFile->WriteString("FT\n"); 	// draw tracks for rectangular pads
							wsprintf(buffer, "%d %d %d %d %d %d\n", nXPos - dx, nYPos + dy, nXPos + dx, nYPos + dy, m_nPPE, 10);
							m_pOutputFile->WriteString(buffer);
							m_pOutputFile->WriteString("FT\n"); 
							wsprintf(buffer, "%d %d %d %d %d %d\n", nXPos + dx, nYPos + dy, nXPos + dx, nYPos - dy, m_nPPE, 10);
							m_pOutputFile->WriteString(buffer);
							m_pOutputFile->WriteString("FT\n"); 
							wsprintf(buffer, "%d %d %d %d %d %d\n", nXPos + dx, nYPos - dy, nXPos - dx, nYPos - dy, m_nPPE, 10);
							m_pOutputFile->WriteString(buffer);
							m_pOutputFile->WriteString("FT\n"); 
							wsprintf(buffer, "%d %d %d %d %d %d\n", nXPos - dx, nYPos - dy, nXPos - dx, nYPos + dy, m_nPPE, 10);
							m_pOutputFile->WriteString(buffer);
						}
						break;
					case 3:
						if(m_bGPRelievePadsVias && m_bGroundOn && (nLayer == m_nGroundSharedLayer || nLayer == 13) && ((nPlane == 1) || (nPlane == 4) || (nPlane == 5))) 
						{
							ASSERT(nXSize = nYSize);
							dx = dy = (nXSize + m_nGPE) / 2;
							dd = (int) ((double) dx * (0.5));
							m_pOutputFile->WriteString("FT\n"); 	// draw tracks for octagonal pads
							wsprintf(buffer, "%d %d %d %d %d %d\n", nXPos - dx, nYPos + dd, nXPos - dd, nYPos + dy, m_nGPE, 9);
							m_pOutputFile->WriteString(buffer);
							m_pOutputFile->WriteString("FT\n"); 
							wsprintf(buffer, "%d %d %d %d %d %d\n", nXPos - dd, nYPos + dy, nXPos + dd, nYPos + dy, m_nGPE, 9);
							m_pOutputFile->WriteString(buffer);
							m_pOutputFile->WriteString("FT\n"); 
							wsprintf(buffer, "%d %d %d %d %d %d\n", nXPos + dd, nYPos + dy, nXPos + dx, nYPos + dd, m_nGPE, 9);
							m_pOutputFile->WriteString(buffer);
							m_pOutputFile->WriteString("FT\n"); 
							wsprintf(buffer, "%d %d %d %d %d %d\n", nXPos + dx, nYPos + dd, nXPos + dx, nYPos - dd, m_nGPE, 9);
							m_pOutputFile->WriteString(buffer);
							m_pOutputFile->WriteString("FT\n"); 
							wsprintf(buffer, "%d %d %d %d %d %d\n", nXPos + dx, nYPos - dd, nXPos + dd, nYPos - dy, m_nGPE, 9);
							m_pOutputFile->WriteString(buffer);
							m_pOutputFile->WriteString("FT\n"); 
							wsprintf(buffer, "%d %d %d %d %d %d\n", nXPos + dd, nYPos - dy, nXPos - dd, nYPos - dy, m_nGPE, 9);
							m_pOutputFile->WriteString(buffer);
							m_pOutputFile->WriteString("FT\n"); 
							wsprintf(buffer, "%d %d %d %d %d %d\n", nXPos - dd, nYPos - dy, nXPos - dx, nYPos - dd, m_nGPE, 9);
							m_pOutputFile->WriteString(buffer);
							m_pOutputFile->WriteString("FT\n"); 
							wsprintf(buffer, "%d %d %d %d %d %d\n", nXPos - dx, nYPos - dd, nXPos - dx, nYPos + dd, m_nGPE, 9);
							m_pOutputFile->WriteString(buffer);
						}
						if(m_bPPRelievePadsVias && m_bPowerOn && (nLayer == m_nPowerSharedLayer || nLayer == 13) && ((nPlane == 1) || (nPlane == 2) || (nPlane == 3))) 
						{
							ASSERT(nXSize = nYSize);
							dx = dy = (nXSize + m_nPPE) / 2;
							dd = (int) ((double) dx * (0.5));
							m_pOutputFile->WriteString("FT\n"); 	// draw tracks for octagonal pads
							wsprintf(buffer, "%d %d %d %d %d %d\n", nXPos - dx, nYPos + dd, nXPos - dd, nYPos + dy, m_nPPE, 10);
							m_pOutputFile->WriteString(buffer);
							m_pOutputFile->WriteString("FT\n"); 
							wsprintf(buffer, "%d %d %d %d %d %d\n", nXPos - dd, nYPos + dy, nXPos + dd, nYPos + dy, m_nPPE, 10);
							m_pOutputFile->WriteString(buffer);
							m_pOutputFile->WriteString("FT\n"); 
							wsprintf(buffer, "%d %d %d %d %d %d\n", nXPos + dd, nYPos + dy, nXPos + dx, nYPos + dd, m_nPPE, 10);
							m_pOutputFile->WriteString(buffer);
							m_pOutputFile->WriteString("FT\n"); 
							wsprintf(buffer, "%d %d %d %d %d %d\n", nXPos + dx, nYPos + dd, nXPos + dx, nYPos - dd, m_nPPE, 10);
							m_pOutputFile->WriteString(buffer);
							m_pOutputFile->WriteString("FT\n"); 
							wsprintf(buffer, "%d %d %d %d %d %d\n", nXPos + dx, nYPos - dd, nXPos + dd, nYPos - dy, m_nPPE, 10);
							m_pOutputFile->WriteString(buffer);
							m_pOutputFile->WriteString("FT\n"); 
							wsprintf(buffer, "%d %d %d %d %d %d\n", nXPos + dd, nYPos - dy, nXPos - dd, nYPos - dy, m_nPPE, 10);
							m_pOutputFile->WriteString(buffer);
							m_pOutputFile->WriteString("FT\n"); 
							wsprintf(buffer, "%d %d %d %d %d %d\n", nXPos - dd, nYPos - dy, nXPos - dx, nYPos - dd, m_nPPE, 10);
							m_pOutputFile->WriteString(buffer);
							m_pOutputFile->WriteString("FT\n"); 
							wsprintf(buffer, "%d %d %d %d %d %d\n", nXPos - dx, nYPos - dd, nXPos - dx, nYPos + dd, m_nPPE, 10);
							m_pOutputFile->WriteString(buffer);
						}
						break;
					case 4:
						if(nXSize == nYSize) 
						{
							if(m_bGPRelievePadsVias && m_bGroundOn && (nLayer == m_nGroundSharedLayer || nLayer == 13) && ((nPlane == 1) || (nPlane == 4) || (nPlane == 5))) 
							{
								m_pOutputFile->WriteString("FA\n");	// draw arcs for circular pads
								wsprintf(buffer, "%d %d %d %d %d %d\n", nXPos, nYPos, (nXSize + m_nGPE) / 2, 15, m_nGPE, 9);
								m_pOutputFile->WriteString(buffer);
							}
							if(m_bPPRelievePadsVias && m_bPowerOn && (nLayer == m_nPowerSharedLayer || nLayer == 13) && ((nPlane == 1) || (nPlane == 2) || (nPlane == 3))) 
							{
								m_pOutputFile->WriteString("FA\n");	// draw arcs for circular pads
								wsprintf(buffer, "%d %d %d %d %d %d\n", nXPos, nYPos, (nXSize + m_nPPE) / 2, 15, m_nPPE, 10);
								m_pOutputFile->WriteString(buffer);
							}
							break;
						}
						if(m_bGPRelievePadsVias && m_bGroundOn && (nLayer == m_nGroundSharedLayer || nLayer == 13) && ((nPlane == 1) || (nPlane == 4) || (nPlane == 5))) 
						{
							if(nXSize > nYSize)  // horizontal pad
							{
								dx = (nXSize - nYSize) / 2;
								dy = nYSize / 2;
								m_pOutputFile->WriteString("FT\n"); 	// draw tracks for rounded rectangle pads
								wsprintf(buffer, "%d %d %d %d %d %d\n", nXPos - dx, nYPos + dy + m_nGPE / 2, nXPos + dx, nYPos + dy + m_nGPE / 2, m_nGPE, 9);
								m_pOutputFile->WriteString(buffer);
								m_pOutputFile->WriteString("FT\n"); 
								wsprintf(buffer, "%d %d %d %d %d %d\n", nXPos - dx, nYPos - dy - m_nGPE / 2, nXPos + dx, nYPos - dy - m_nGPE / 2, m_nGPE, 9);
								m_pOutputFile->WriteString(buffer);
								m_pOutputFile->WriteString("FA\n");	// draw arcs for circular pads
								wsprintf(buffer, "%d %d %d %d %d %d\n", nXPos - dx, nYPos, dy + m_nGPE / 2, 6, m_nGPE, 9);
								m_pOutputFile->WriteString(buffer);
								m_pOutputFile->WriteString("FA\n");	// draw arcs for circular pads
								wsprintf(buffer, "%d %d %d %d %d %d\n", nXPos + dx, nYPos, dy + m_nGPE / 2, 9, m_nGPE, 9);
								m_pOutputFile->WriteString(buffer);
							}
							if(nXSize < nYSize)  // vertical pad
							{
								dx = nXSize / 2;
								dy = (nYSize - nXSize) / 2;
								m_pOutputFile->WriteString("FT\n"); 	// draw tracks for rounded rectangle pads
								wsprintf(buffer, "%d %d %d %d %d %d\n", nXPos - dx - m_nGPE / 2, nYPos - dy, nXPos - dx  - m_nGPE / 2, nYPos + dy, m_nGPE, 9);
								m_pOutputFile->WriteString(buffer);
								m_pOutputFile->WriteString("FT\n"); 
								wsprintf(buffer, "%d %d %d %d %d %d\n", nXPos + dx + m_nGPE / 2, nYPos - dy, nXPos + dx  + m_nGPE / 2, nYPos + dy, m_nGPE, 9);
								m_pOutputFile->WriteString(buffer);
								m_pOutputFile->WriteString("FA\n");	// draw arcs for circular pads
								wsprintf(buffer, "%d %d %d %d %d %d\n", nXPos, nYPos - dy, dx + m_nGPE / 2, 12, m_nGPE, 9);
								m_pOutputFile->WriteString(buffer);
								m_pOutputFile->WriteString("FA\n");	// draw arcs for circular pads
								wsprintf(buffer, "%d %d %d %d %d %d\n", nXPos, nYPos + dy, dx + m_nGPE / 2, 3, m_nGPE, 9);
								m_pOutputFile->WriteString(buffer);
							}
						}
						if(m_bPPRelievePadsVias && m_bPowerOn && (nLayer == m_nPowerSharedLayer || nLayer == 13) && ((nPlane == 1) || (nPlane == 2) || (nPlane == 3))) 
						{
							if(nXSize > nYSize)  // horizontal pad
							{
								dx = (nXSize - nYSize) / 2;
								dy = nYSize / 2;
								m_pOutputFile->WriteString("FT\n"); 	// draw tracks for rounded rectangle pads
								wsprintf(buffer, "%d %d %d %d %d %d\n", nXPos - dx, nYPos + dy + m_nPPE / 2, nXPos + dx, nYPos + dy + m_nPPE / 2, m_nPPE, 10);
								m_pOutputFile->WriteString(buffer);
								m_pOutputFile->WriteString("FT\n"); 
								wsprintf(buffer, "%d %d %d %d %d %d\n", nXPos - dx, nYPos - dy - m_nPPE / 2, nXPos + dx, nYPos - dy - m_nPPE / 2, m_nPPE, 10);
								m_pOutputFile->WriteString(buffer);
								m_pOutputFile->WriteString("FA\n");	// draw arcs for circular pads
								wsprintf(buffer, "%d %d %d %d %d %d\n", nXPos - dx, nYPos, dy + m_nPPE / 2, 6, m_nPPE, 10);
								m_pOutputFile->WriteString(buffer);
								m_pOutputFile->WriteString("FA\n");	// draw arcs for circular pads
								wsprintf(buffer, "%d %d %d %d %d %d\n", nXPos + dx, nYPos, dy + m_nPPE / 2, 9, m_nPPE, 10);
								m_pOutputFile->WriteString(buffer);
							}
							if(nXSize < nYSize)  // horizontal pad
							{
								dx = nXSize / 2;
								dy = (nYSize - nXSize) / 2;
								m_pOutputFile->WriteString("FT\n"); 	// draw tracks for rounded rectangle pads
								wsprintf(buffer, "%d %d %d %d %d %d\n", nXPos - dx - m_nPPE / 2, nYPos - dy, nXPos - dx  - m_nPPE / 2, nYPos + dy, m_nPPE, 10);
								m_pOutputFile->WriteString(buffer);
								m_pOutputFile->WriteString("FT\n"); 
								wsprintf(buffer, "%d %d %d %d %d %d\n", nXPos + dx + m_nPPE / 2, nYPos - dy, nXPos + dx  + m_nPPE / 2, nYPos + dy, m_nPPE, 10);
								m_pOutputFile->WriteString(buffer);
								m_pOutputFile->WriteString("FA\n");	// draw arcs for circular pads
								wsprintf(buffer, "%d %d %d %d %d %d\n", nXPos, nYPos - dy, dx + m_nPPE / 2, 12, m_nPPE, 10);
								m_pOutputFile->WriteString(buffer);
								m_pOutputFile->WriteString("FA\n");	// draw arcs for circular pads
								wsprintf(buffer, "%d %d %d %d %d %d\n", nXPos, nYPos + dy, dx + m_nPPE / 2, 3, m_nPPE, 10);
								m_pOutputFile->WriteString(buffer);
							}
						}
						break;
					default:
						break;
					} // end pad switch
				}
			}
			else if((cs.Left(2) == "CA"))    // arcs
			{
				m_pInputFile->ReadString(buffer, 40);
				m_pOutputFile->WriteString(buffer);
				sscanf(buffer, "%d %d %d %d %d %d", &p1, &p2, &p3, &p4, &nXSize, &p6);
				if(p6 == m_nGroundSharedLayer && m_bGroundOn)
				{
					m_pOutputFile->WriteString("CA\n");
					wsprintf(buffer, "%d %d %d %d %d %d\n", p1, p2, p3, p4, nXSize + 2 * m_nGPE, 9);
					m_pOutputFile->WriteString(buffer);
				}				
				if(p6 == m_nPowerSharedLayer && m_bPowerOn)
				{
					m_pOutputFile->WriteString("CA\n");
					wsprintf(buffer, "%d %d %d %d %d %d\n", p1, p2, p3, p4, nXSize + 2 * m_nPPE, 10);
					m_pOutputFile->WriteString(buffer);
				}				
				 
			} 
			else if((cs.Left(2) == "CT"))     // tracks
			{		
				m_pInputFile->ReadString(buffer, 40);
				m_pOutputFile->WriteString(buffer);
				sscanf(buffer, "%d %d %d %d %d %d %d", &p1, &p2, &p3, &p4, &nXSize, &p6, &p7);
				if(p6 == m_nGroundSharedLayer && m_bGroundOn)
				{
					m_pOutputFile->WriteString("CT\n");
					wsprintf(buffer, "%d %d %d %d %d %d %d\n", p1, p2, p3, p4, nXSize + 2 * m_nGPE, 9, p7);
					m_pOutputFile->WriteString(buffer);
				}				
				if(p6 == m_nPowerSharedLayer && m_bPowerOn)
				{
					m_pOutputFile->WriteString("CT\n");
					wsprintf(buffer, "%d %d %d %d %d %d %d\n", p1, p2, p3, p4, nXSize + 2 * m_nPPE, 10, p7);
					m_pOutputFile->WriteString(buffer);
				}				
			}
			else if((cs.Left(2) == "CV"))     // vias
			{		
				m_pInputFile->ReadString(buffer, 40);
				m_pOutputFile->WriteString(buffer);
				sscanf(buffer, "%d %d %d %d", &p1, &p2, &nXSize, &p3);
				if(m_bGroundOn && m_bGPRelievePadsVias)
				{
					m_pOutputFile->WriteString("CA\n");
					wsprintf(buffer, "%d %d %d %d %d %d\n", p1, p2, ((nXSize / 2) + m_nGPE) / 2, 15, (nXSize / 2) + m_nGPE, 9);
					m_pOutputFile->WriteString(buffer);
				}				
				if(m_bPowerOn && m_bPPRelievePadsVias)
				{
					m_pOutputFile->WriteString("CA\n");
					wsprintf(buffer, "%d %d %d %d %d %d\n", p1, p2, ((nXSize / 2) + m_nPPE) / 2, 15, (nXSize / 2) + m_nPPE, 10);
					m_pOutputFile->WriteString(buffer);
				}				
			}
			else if((cs.Left(2) == "CP"))     // pads
			{		
				m_pInputFile->ReadString(buffer, 40);
				m_pOutputFile->WriteString(buffer);		// copy the original
				sscanf(buffer, "%d %d %d %d %d %d %d %d", &nXPos, &nYPos, &nXSize, &nYSize, &nShape, &nHole, &nPlane, &nLayer);
				m_pInputFile->ReadString(buffer, 40);	// don't forget the name of the pad!
				m_pOutputFile->WriteString(buffer);		// copy the original
				if((m_bGroundOn || m_bPowerOn) && (nLayer == 13 || nLayer == m_nGroundSharedLayer || nLayer == m_nPowerSharedLayer) && (m_bGPRelievePadsVias || m_bPPRelievePadsVias))
				{
					switch(nShape)
					{
					case 1:
						if(m_bGPRelievePadsVias && m_bGroundOn && (nLayer == m_nGroundSharedLayer || nLayer == 13) && ((nPlane == 1) || (nPlane == 4) || (nPlane == 5))) {
							m_pOutputFile->WriteString("CA\n");	// draw arcs for circular pads
							wsprintf(buffer, "%d %d %d %d %d %d\n", nXPos, nYPos, (nXSize + m_nGPE) / 2, 15, m_nGPE, 9);
							m_pOutputFile->WriteString(buffer);
						}
						if(m_bPPRelievePadsVias && m_bPowerOn && (nLayer == m_nPowerSharedLayer || nLayer == 13) && ((nPlane == 1) || (nPlane == 2) || (nPlane == 3))) {
							m_pOutputFile->WriteString("CA\n");	// draw arcs for circular pads
							wsprintf(buffer, "%d %d %d %d %d %d\n", nXPos, nYPos, (nXSize + m_nPPE) / 2, 15, m_nPPE, 10);
							m_pOutputFile->WriteString(buffer);
						}
						break;
					case 2:
						if(m_bGPRelievePadsVias && m_bGroundOn && (nLayer == m_nGroundSharedLayer || nLayer == 13) && ((nPlane == 1) || (nPlane == 4) || (nPlane == 5))) {
							dx = (nXSize + m_nGPE) / 2;
							dy = (nYSize + m_nGPE) / 2;
							m_pOutputFile->WriteString("CT\n"); 	// draw tracks for rectangular pads
							wsprintf(buffer, "%d %d %d %d %d %d\n", nXPos - dx, nYPos + dy, nXPos + dx, nYPos + dy, m_nGPE, 9);
							m_pOutputFile->WriteString(buffer);
							m_pOutputFile->WriteString("CT\n"); 
							wsprintf(buffer, "%d %d %d %d %d %d\n", nXPos + dx, nYPos + dy, nXPos + dx, nYPos - dy, m_nGPE, 9);
							m_pOutputFile->WriteString(buffer);
							m_pOutputFile->WriteString("CT\n"); 
							wsprintf(buffer, "%d %d %d %d %d %d\n", nXPos + dx, nYPos - dy, nXPos - dx, nYPos - dy, m_nGPE, 9);
							m_pOutputFile->WriteString(buffer);
							m_pOutputFile->WriteString("CT\n"); 
							wsprintf(buffer, "%d %d %d %d %d %d\n", nXPos - dx, nYPos - dy, nXPos - dx, nYPos + dy, m_nGPE, 9);
							m_pOutputFile->WriteString(buffer);
						}
						if(m_bPPRelievePadsVias && m_bPowerOn && (nLayer == m_nPowerSharedLayer || nLayer == 13) && ((nPlane == 1) || (nPlane == 2) || (nPlane == 3))) {
							dx = (nXSize + m_nPPE) / 2;
							dy = (nYSize + m_nPPE) / 2;
							m_pOutputFile->WriteString("CT\n"); 	// draw tracks for rectangular pads
							wsprintf(buffer, "%d %d %d %d %d %d\n", nXPos - dx, nYPos + dy, nXPos + dx, nYPos + dy, m_nPPE, 10);
							m_pOutputFile->WriteString(buffer);
							m_pOutputFile->WriteString("CT\n"); 
							wsprintf(buffer, "%d %d %d %d %d %d\n", nXPos + dx, nYPos + dy, nXPos + dx, nYPos - dy, m_nPPE, 10);
							m_pOutputFile->WriteString(buffer);
							m_pOutputFile->WriteString("CT\n"); 
							wsprintf(buffer, "%d %d %d %d %d %d\n", nXPos + dx, nYPos - dy, nXPos - dx, nYPos - dy, m_nPPE, 10);
							m_pOutputFile->WriteString(buffer);
							m_pOutputFile->WriteString("CT\n"); 
							wsprintf(buffer, "%d %d %d %d %d %d\n", nXPos - dx, nYPos - dy, nXPos - dx, nYPos + dy, m_nPPE, 10);
							m_pOutputFile->WriteString(buffer);
						}
						break;
					case 3:
						if(m_bGPRelievePadsVias && m_bGroundOn && (nLayer == m_nGroundSharedLayer || nLayer == 13) && ((nPlane == 1) || (nPlane == 4) || (nPlane == 5))) 
						{
							ASSERT(nXSize = nYSize);
							dx = dy = (nXSize + m_nGPE) / 2;
							dd = (int) ((double) dx * (0.5));
							m_pOutputFile->WriteString("CT\n"); 	// draw tracks for octagonal pads
							wsprintf(buffer, "%d %d %d %d %d %d\n", nXPos - dx, nYPos + dd, nXPos - dd, nYPos + dy, m_nGPE, 9);
							m_pOutputFile->WriteString(buffer);
							m_pOutputFile->WriteString("CT\n"); 
							wsprintf(buffer, "%d %d %d %d %d %d\n", nXPos - dd, nYPos + dy, nXPos + dd, nYPos + dy, m_nGPE, 9);
							m_pOutputFile->WriteString(buffer);
							m_pOutputFile->WriteString("CT\n"); 
							wsprintf(buffer, "%d %d %d %d %d %d\n", nXPos + dd, nYPos + dy, nXPos + dx, nYPos + dd, m_nGPE, 9);
							m_pOutputFile->WriteString(buffer);
							m_pOutputFile->WriteString("CT\n"); 
							wsprintf(buffer, "%d %d %d %d %d %d\n", nXPos + dx, nYPos + dd, nXPos + dx, nYPos - dd, m_nGPE, 9);
							m_pOutputFile->WriteString(buffer);
							m_pOutputFile->WriteString("CT\n"); 
							wsprintf(buffer, "%d %d %d %d %d %d\n", nXPos + dx, nYPos - dd, nXPos + dd, nYPos - dy, m_nGPE, 9);
							m_pOutputFile->WriteString(buffer);
							m_pOutputFile->WriteString("CT\n"); 
							wsprintf(buffer, "%d %d %d %d %d %d\n", nXPos + dd, nYPos - dy, nXPos - dd, nYPos - dy, m_nGPE, 9);
							m_pOutputFile->WriteString(buffer);
							m_pOutputFile->WriteString("CT\n"); 
							wsprintf(buffer, "%d %d %d %d %d %d\n", nXPos - dd, nYPos - dy, nXPos - dx, nYPos - dd, m_nGPE, 9);
							m_pOutputFile->WriteString(buffer);
							m_pOutputFile->WriteString("CT\n"); 
							wsprintf(buffer, "%d %d %d %d %d %d\n", nXPos - dx, nYPos - dd, nXPos - dx, nYPos + dd, m_nGPE, 9);
							m_pOutputFile->WriteString(buffer);
						}
						if(m_bPPRelievePadsVias && m_bPowerOn && (nLayer == m_nPowerSharedLayer || nLayer == 13) && ((nPlane == 1) || (nPlane == 2) || (nPlane == 3))) 
						{
							ASSERT(nXSize = nYSize);
							dx = dy = (nXSize + m_nPPE) / 2;
							dd = (int) ((double) dx * (0.5));
							m_pOutputFile->WriteString("CT\n"); 	// draw tracks for octagonal pads
							wsprintf(buffer, "%d %d %d %d %d %d\n", nXPos - dx, nYPos + dd, nXPos - dd, nYPos + dy, m_nPPE, 10);
							m_pOutputFile->WriteString(buffer);
							m_pOutputFile->WriteString("CT\n"); 
							wsprintf(buffer, "%d %d %d %d %d %d\n", nXPos - dd, nYPos + dy, nXPos + dd, nYPos + dy, m_nPPE, 10);
							m_pOutputFile->WriteString(buffer);
							m_pOutputFile->WriteString("CT\n"); 
							wsprintf(buffer, "%d %d %d %d %d %d\n", nXPos + dd, nYPos + dy, nXPos + dx, nYPos + dd, m_nPPE, 10);
							m_pOutputFile->WriteString(buffer);
							m_pOutputFile->WriteString("CT\n"); 
							wsprintf(buffer, "%d %d %d %d %d %d\n", nXPos + dx, nYPos + dd, nXPos + dx, nYPos - dd, m_nPPE, 10);
							m_pOutputFile->WriteString(buffer);
							m_pOutputFile->WriteString("CT\n"); 
							wsprintf(buffer, "%d %d %d %d %d %d\n", nXPos + dx, nYPos - dd, nXPos + dd, nYPos - dy, m_nPPE, 10);
							m_pOutputFile->WriteString(buffer);
							m_pOutputFile->WriteString("CT\n"); 
							wsprintf(buffer, "%d %d %d %d %d %d\n", nXPos + dd, nYPos - dy, nXPos - dd, nYPos - dy, m_nPPE, 10);
							m_pOutputFile->WriteString(buffer);
							m_pOutputFile->WriteString("CT\n"); 
							wsprintf(buffer, "%d %d %d %d %d %d\n", nXPos - dd, nYPos - dy, nXPos - dx, nYPos - dd, m_nPPE, 10);
							m_pOutputFile->WriteString(buffer);
							m_pOutputFile->WriteString("CT\n"); 
							wsprintf(buffer, "%d %d %d %d %d %d\n", nXPos - dx, nYPos - dd, nXPos - dx, nYPos + dd, m_nPPE, 10);
							m_pOutputFile->WriteString(buffer);
						}
						break;
					case 4:
						if(nXSize == nYSize) 
						{
							if(m_bGPRelievePadsVias && m_bGroundOn && (nLayer == m_nGroundSharedLayer || nLayer == 13) && ((nPlane == 1) || (nPlane == 4) || (nPlane == 5))) 
							{
								m_pOutputFile->WriteString("CA\n");	// draw arcs for circular pads
								wsprintf(buffer, "%d %d %d %d %d %d\n", nXPos, nYPos, (nXSize + m_nGPE) / 2, 15, m_nGPE, 9);
								m_pOutputFile->WriteString(buffer);
							}
							if(m_bPPRelievePadsVias && m_bPowerOn && (nLayer == m_nPowerSharedLayer || nLayer == 13) && ((nPlane == 1) || (nPlane == 2) || (nPlane == 3))) 
							{
								m_pOutputFile->WriteString("CA\n");	// draw arcs for circular pads
								wsprintf(buffer, "%d %d %d %d %d %d\n", nXPos, nYPos, (nXSize + m_nPPE) / 2, 15, m_nPPE, 10);
								m_pOutputFile->WriteString(buffer);
							}
							break;
						}
						if(m_bGPRelievePadsVias && m_bGroundOn && (nLayer == m_nGroundSharedLayer || nLayer == 13) && ((nPlane == 1) || (nPlane == 4) || (nPlane == 5))) 
						{
							if(nXSize > nYSize)  // horizontal pad
							{
								dx = (nXSize - nYSize) / 2;
								dy = nYSize / 2;
								m_pOutputFile->WriteString("CT\n"); 	// draw tracks for rounded rectangle pads
								wsprintf(buffer, "%d %d %d %d %d %d\n", nXPos - dx, nYPos + dy + m_nGPE / 2, nXPos + dx, nYPos + dy + m_nGPE / 2, m_nGPE, 9);
								m_pOutputFile->WriteString(buffer);
								m_pOutputFile->WriteString("CT\n"); 
								wsprintf(buffer, "%d %d %d %d %d %d\n", nXPos - dx, nYPos - dy - m_nGPE / 2, nXPos + dx, nYPos - dy - m_nGPE / 2, m_nGPE, 9);
								m_pOutputFile->WriteString(buffer);
								m_pOutputFile->WriteString("CA\n");	// draw arcs for circular pads
								wsprintf(buffer, "%d %d %d %d %d %d\n", nXPos - dx, nYPos, dy + m_nGPE / 2, 6, m_nGPE, 9);
								m_pOutputFile->WriteString(buffer);
								m_pOutputFile->WriteString("CA\n");	// draw arcs for circular pads
								wsprintf(buffer, "%d %d %d %d %d %d\n", nXPos + dx, nYPos, dy + m_nGPE / 2, 9, m_nGPE, 9);
								m_pOutputFile->WriteString(buffer);
							}
							if(nXSize < nYSize)  // horizontal pad
							{
								dx = nXSize / 2;
								dy = (nYSize - nXSize) / 2;
								m_pOutputFile->WriteString("CT\n"); 	// draw tracks for rounded rectangle pads
								wsprintf(buffer, "%d %d %d %d %d %d\n", nXPos - dx - m_nGPE / 2, nYPos - dy, nXPos - dx  - m_nGPE / 2, nYPos + dy, m_nGPE, 9);
								m_pOutputFile->WriteString(buffer);
								m_pOutputFile->WriteString("CT\n"); 
								wsprintf(buffer, "%d %d %d %d %d %d\n", nXPos + dx + m_nGPE / 2, nYPos - dy, nXPos + dx  + m_nGPE / 2, nYPos + dy, m_nGPE, 9);
								m_pOutputFile->WriteString(buffer);
								m_pOutputFile->WriteString("CA\n");	// draw arcs for circular pads
								wsprintf(buffer, "%d %d %d %d %d %d\n", nXPos, nYPos - dy, dx + m_nGPE / 2, 12, m_nGPE, 9);
								m_pOutputFile->WriteString(buffer);
								m_pOutputFile->WriteString("CA\n");	// draw arcs for circular pads
								wsprintf(buffer, "%d %d %d %d %d %d\n", nXPos, nYPos + dy, dx + m_nGPE / 2, 3, m_nGPE, 9);
								m_pOutputFile->WriteString(buffer);
							}
						}
						if(m_bPPRelievePadsVias && m_bPowerOn && (nLayer == m_nPowerSharedLayer || nLayer == 13) && ((nPlane == 1) || (nPlane == 2) || (nPlane == 3))) 
						{
							if(nXSize > nYSize)  // horizontal pad
							{
								dx = (nXSize - nYSize) / 2;
								dy = nYSize / 2;
								m_pOutputFile->WriteString("CT\n"); 	// draw tracks for rounded rectangle pads
								wsprintf(buffer, "%d %d %d %d %d %d\n", nXPos - dx, nYPos + dy + m_nPPE / 2, nXPos + dx, nYPos + dy + m_nPPE / 2, m_nPPE, 10);
								m_pOutputFile->WriteString(buffer);
								m_pOutputFile->WriteString("CT\n"); 
								wsprintf(buffer, "%d %d %d %d %d %d\n", nXPos - dx, nYPos - dy - m_nPPE / 2, nXPos + dx, nYPos - dy - m_nPPE / 2, m_nPPE, 10);
								m_pOutputFile->WriteString(buffer);
								m_pOutputFile->WriteString("CA\n");	// draw arcs for circular pads
								wsprintf(buffer, "%d %d %d %d %d %d\n", nXPos - dx, nYPos, dy + m_nPPE / 2, 6, m_nPPE, 10);
								m_pOutputFile->WriteString(buffer);
								m_pOutputFile->WriteString("CA\n");	// draw arcs for circular pads
								wsprintf(buffer, "%d %d %d %d %d %d\n", nXPos + dx, nYPos, dy + m_nPPE / 2, 9, m_nPPE, 10);
								m_pOutputFile->WriteString(buffer);
							}
							if(nXSize < nYSize)  // horizontal pad
							{
								dx = nXSize / 2;
								dy = (nYSize - nXSize) / 2;
								m_pOutputFile->WriteString("CT\n"); 	// draw tracks for rounded rectangle pads
								wsprintf(buffer, "%d %d %d %d %d %d\n", nXPos - dx - m_nPPE / 2, nYPos - dy, nXPos - dx  - m_nPPE / 2, nYPos + dy, m_nPPE, 10);
								m_pOutputFile->WriteString(buffer);
								m_pOutputFile->WriteString("CT\n"); 
								wsprintf(buffer, "%d %d %d %d %d %d\n", nXPos + dx + m_nPPE / 2, nYPos - dy, nXPos + dx  + m_nPPE / 2, nYPos + dy, m_nPPE, 10);
								m_pOutputFile->WriteString(buffer);
								m_pOutputFile->WriteString("CA\n");	// draw arcs for circular pads
								wsprintf(buffer, "%d %d %d %d %d %d\n", nXPos, nYPos - dy, dx + m_nPPE / 2, 12, m_nPPE, 10);
								m_pOutputFile->WriteString(buffer);
								m_pOutputFile->WriteString("CA\n");	// draw arcs for circular pads
								wsprintf(buffer, "%d %d %d %d %d %d\n", nXPos, nYPos + dy, dx + m_nPPE / 2, 3, m_nPPE, 10);
								m_pOutputFile->WriteString(buffer);
							}
						}
						break;
					default:
						break;
					} // end pad switch
				}
			}
		} 
	} while(1);

}

void CGrounder32::OnButtonHelp() 
{
	CHelp dlg;
	dlg.DoModal();

}
